package script

import util.CIProperties
import util.DockerImageNames
import util.DockerTags
import util.Dockerfiles

/**
 * Скрип для деплоя докер образа на сервер посредством обновления docker-compose файла и его перезапуска.
 */

@GrabResolver(name='gitea', root='https://git.tswf.io/api/packages/public-repos/maven')
@Grapes([
        @Grab(group='io.tswf.groovy.better-groovy', module='better-groovy-scripting-shell', version='2.0.2-SNAPSHOT', changing = true),
        @Grab(group='io.tswf.groovy.better-groovy', module='better-groovy-scripting-gitea', version='2.0.2-SNAPSHOT', changing = true)
])
import util.GitTags
import util.GlobalProperties
import util.ScriptLog

GlobalProperties.loadGlobalProperties()

println """
#############################################################################
                       DEPLOY via docker-compose script
#############################################################################
"""

def deployGitTags = GitTags.getDeployTags()

if (deployGitTags.isNullOrEmpty()) {
    ScriptLog.printf "There is no tags to remote restart."
    System.exit(1)
}

def deployDockerfiles = Dockerfiles.getDeployDockerfiles()

def dockerComposeBaseCommand = CIProperties.findProperty("deploy.docker-compose.base-command")
    .orElse("docker compose")

def dockerComposeDirectory = CIProperties.getProperty("deploy.docker-compose.dir")

def dockerComposeFileName = CIProperties.findProperty("deploy.docker-compose.filename")
    .orElse("docker-compose.yml")

def sshDeployProfile = CIProperties.findProperty("deploy.ssh.profile")
    .orElse("deploy")

if (deployGitTags.size() > 1) {
    throw new IllegalStateException("Can not deploy more than one git tag via docker-compose. Found multiple deploy tags: ${deployGitTags}")
}

def deployGitTag = deployGitTags.first()

ScriptLog.printf "Starting, total ${deployDockerfiles.size()} dockerfiles marked to be deployed... (tag: ${deployGitTag}, dockerfiles: ${deployDockerfiles})"

for (def dockerfileName : deployDockerfiles) {
    ScriptLog.printf "Deploying image for dockerfile named '${dockerfileName}' via ssh profile ${sshDeployProfile}"

    def dockerImageToUpdateTagBaseName = DockerImageNames.getImageName(dockerfileName)

    ScriptLog.printf "Docker image ${dockerImageToUpdateTagBaseName} base name will be used for dockerfile named '${dockerfileName}'"

    def dockerReleaseTagToSetup = DockerTags.getDockerTagPostfixForDockerfile(
        GitTags.sanitizeTagFromPrefixes(deployGitTag, GitTags.deployPrefixes)
    )

    ScriptLog.printf "Updating docker-compose image tag to '${dockerReleaseTagToSetup}' in service '${dockerImageToUpdateTagBaseName}' in file '${dockerComposeFileName}' located in directory '${dockerComposeDirectory}'..."

    sh """ ssh -tt ${sshDeployProfile} "\\
       cd '${dockerComposeDirectory}' && \\
       sed -i 's|image: ${dockerImageToUpdateTagBaseName}:.*|image: ${dockerImageToUpdateTagBaseName}:${dockerReleaseTagToSetup}|g' '${dockerComposeFileName}'\\
       "
    """

    ScriptLog.printf "Service tag updated, restarting compose..."

    sh """ ssh -tt ${sshDeployProfile} "\\
       cd '${dockerComposeDirectory}' && \\
       ${dockerComposeBaseCommand} -f '${dockerComposeFileName}' up -d \\
       "
    """

    ScriptLog.printf "Deploying of docker tag '${dockerReleaseTagToSetup}' successfully completed!"
}